/**
 * Copyright (c) 2018 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * Notwithstanding the foregoing, you may not use, copy, modify, merge, publish,
 * distribute, sublicense, create a derivative work, and/or sell copies of the
 * Software in any work that is designed, intended, or marketed for pedagogical or
 * instructional purposes related to programming, coding, application development,
 * or information technology.  Permission for such use, copying, modification,
 * merger, publication, distribution, sublicensing, creation of derivative works,
 * or sale is expressly withheld.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import MetalKit

class Model: Node {
  /*
   static var defaultVertexDescriptor: MDLVertexDescriptor = {
   let vertexDescriptor = MDLVertexDescriptor()
   vertexDescriptor.attributes[0] = MDLVertexAttribute(name: MDLVertexAttributePosition,
   format: .float3,
   offset: 0, bufferIndex: 0)
   // add the normal attribute here
   vertexDescriptor.attributes[1] = MDLVertexAttribute(name: MDLVertexAttributeNormal,
   format: .float3,
   offset: 12, bufferIndex: 0)
   vertexDescriptor.layouts[0] = MDLVertexBufferLayout(stride: 12)
   return vertexDescriptor
   }()
   */
  let vertexBuffer: MTLBuffer
  let pipelineState: MTLRenderPipelineState
  let mesh: MTKMesh
  let submeshes: [Submesh]
  
  init(name: String) {
    
    
    
    
    let allocator = MTKMeshBufferAllocator(device: Renderer.device)
    /*
     let assetURL = Bundle.main.url(forResource: name, withExtension: "obj")!
     let asset = MDLAsset(url: assetURL, vertexDescriptor: Model.defaultVertexDescriptor,
     bufferAllocator: allocator)
     let mdlMesh = asset.object(at: 0) as! MDLMesh
     
     */
    
    
    
    //let allocator = MTKMeshBufferAllocator(device: Renderer.device)
    let vertexData: [float3] = [
      float3(-0.250, 0.750, 0.50),
      float3(0, 0, 0),
      float3(-0.250, -0.750, 0.50),
      float3(0, 0, 0),
      float3(0.250, -0.750, 0.50),
      float3(0, 0, 0),
      float3( 0.250, 0.750, 0.50),
      float3(0, 0, 0),
    ]
    
    
    let indices: [UInt16] = [
      // bottom front, top front
      0,1,2,
      2,3,0//,  // FRONT
      // 7, 5, 4,  6,5,7  //BACK
      // 0, 1, 5,  0, 5, 4,  //LEFT
      // 3, 2, 6,  3, 6, 7,  //RIGHT
      // 4, 0, 3,  4, 3, 7, // TOP
      // 1, 5, 2,  2, 5, 6, // BOTTOM
      
      
    ]
    
    
    
    let mdlMeshVertexBuffer = allocator.newBuffer(MemoryLayout<float3>.stride * vertexData.count, type: .vertex)
    let vertexMap = mdlMeshVertexBuffer.map()
    vertexMap.bytes.assumingMemoryBound(to: float3.self).assign(from: vertexData, count: vertexData.count)
    
    let indexBuffer = allocator.newBuffer(MemoryLayout<UInt16>.stride * indices.count, type: .index)
    let indexMap = indexBuffer.map()
    indexMap.bytes.assumingMemoryBound(to: UInt16.self).assign(from: indices, count: indices.count)
    
    let scatteringFunction = MDLScatteringFunction()
    let material = MDLMaterial(name: "baseMaterial", scatteringFunction: scatteringFunction)
    //material.setTextures(textures: [.baseColor:"gray_material.jpg"])
    
    
    let submesh = MDLSubmesh(indexBuffer: indexBuffer,
                             indexCount: indices.count,
                             indexType: .uInt16,
                             geometryType: .triangles,
                             material: material)
    
    let vertexDescriptor = MDLVertexDescriptor()
    vertexDescriptor.attributes[0] = MDLVertexAttribute(name: MDLVertexAttributePosition,
                                                        format: .float3,
                                                        offset: 0,
                                                        bufferIndex: 0)
    
    vertexDescriptor.attributes[1] = MDLVertexAttribute(name: MDLVertexAttributeNormal,
                                                        format: .float3,
                                                        offset: MemoryLayout<float3>.stride,
                                                        bufferIndex: 0)

    vertexDescriptor.layouts[0] = MDLVertexBufferLayout(stride: MemoryLayout<float3>.stride*2)
    
    
    
    let mdlMesh = MDLMesh(vertexBuffer: mdlMeshVertexBuffer,
                          vertexCount: vertexData.count/2,
                          descriptor: vertexDescriptor,
                          submeshes: [submesh])
    
    
    mdlMesh.addNormals(withAttributeNamed: MDLVertexAttributeNormal, creaseThreshold: 0.99)
    
    
    
    
    
    
    
    /*
     let mdlMesh = MDLMesh(boxWithExtent: [0.5,1.5,1],
     segments: [1, 1, 1],
     inwardNormals: false,
     // cap: true,
     geometryType: .triangles,
     allocator: allocator)
     
     */
    
    print (mdlMesh.vertexBuffers[0].length)
    Model.printFloat3Buffer(mdlMesh)
    
    //for i in 0..<mdlMesh.vertexCount{
    print ("first elament  is: ")
    print (mdlMesh.vertexBuffers[0])
    //
    
    
    print ("Getting metal mesh from obj mdlMesh")
    let mesh = try! MTKMesh(mesh: mdlMesh, device: Renderer.device)
    self.mesh = mesh
    print ("Assigning vertext Buffer now")
    vertexBuffer = mesh.vertexBuffers[0].buffer
    print (vertexBuffer)
    
    submeshes = mdlMesh.submeshes?.enumerated().compactMap {index, submesh in
      (submesh as? MDLSubmesh).map {
        Submesh(submesh: mesh.submeshes[index],
                mdlSubmesh: $0)
      }
      } ?? []
    
    print ("Getting sub-meshes count is \(submeshes.count)")
    
    
    
    pipelineState = Model.buildPipelineState(vertexDescriptor: mdlMesh.vertexDescriptor)
    print ("Setting pipeline state")
    super.init()
  }
  
  private static func buildPipelineState(vertexDescriptor: MDLVertexDescriptor) -> MTLRenderPipelineState {
    let library = Renderer.library
    let vertexFunction = library?.makeFunction(name: "vertex_main")
    let fragmentFunction = library?.makeFunction(name: "fragment_main")
    
    var pipelineState: MTLRenderPipelineState
    let pipelineDescriptor = MTLRenderPipelineDescriptor()
    pipelineDescriptor.vertexFunction = vertexFunction
    pipelineDescriptor.fragmentFunction = fragmentFunction
    pipelineDescriptor.vertexDescriptor = MTKMetalVertexDescriptorFromModelIO(vertexDescriptor)
    pipelineDescriptor.colorAttachments[0].pixelFormat = Renderer.colorPixelFormat
    pipelineDescriptor.depthAttachmentPixelFormat = .depth32Float
    do {
      pipelineState = try Renderer.device.makeRenderPipelineState(descriptor: pipelineDescriptor)
    } catch let error {
      fatalError(error.localizedDescription)
    }
    return pipelineState
  }
  
  private static func printFloat3Buffer(_ mdlMesh: MDLMesh){
    let vbuff = mdlMesh.vertexBuffers[0]
    let bufmap = mdlMesh.vertexBuffers[0].map()
    
    let layout = mdlMesh.vertexDescriptor.layouts.firstObject as! MDLVertexBufferLayout
    let stride = layout.stride
    print ("Stride is \(stride)")
    print ("Vbuff lenght  = \(vbuff.length)")
    print("vertexCount = ", mdlMesh.vertexCount)
    let tmpdata = Data(bytes: bufmap.bytes, count: vbuff.length)
        for i in 0..<mdlMesh.vertexCount {
      var v = float3(0)
      let buffer = UnsafeMutableBufferPointer(start: &v, count: 1)
      let start = i*stride
      tmpdata.copyBytes(to: buffer, from: start..<start+MemoryLayout<Float>.size * 3)
      print(v)
    }
    
  }
}


